classdef genericReadAsFrames
    % Object that contains various info about microscopy images
    % obtained from metadata file
    properties (Access = private)
        positions        = {};
        positionsMap     = [];
        uniqueTimepoints = {};
        acquireTimes     = {};
        channels         = {};
        imgNrAdapter     = {};
        exposureTimes    = {};
        absPath          = '';
        prefix           = 'img_';
        nameSep          = '_';
        posChCounters    = [];
        posFrameCounters = [];
        croppingRegions  = {};
    end
    
    %% Private methods
    methods (Access = private)
        function img = getImg(obj, posIdx, frameIdx, chanIdx)
            imgNr = obj.imgNrAdapter{posIdx}(frameIdx,chanIdx);
            
            if (imgNr ~= -1 )
                filename = [obj.absPath filesep obj.positions{posIdx}...
                            filesep obj.channels{chanIdx} filesep obj.prefix...
                            obj.nameSep sprintf('%09d', imgNr)...
                            obj.nameSep...
                            num2str(obj.exposureTimes{posIdx}(frameIdx,chanIdx))...
                            obj.nameSep sprintf('%03d', 0)...
                            '.tiff'];
                
                img = uint16(imread(filename));
            else
                img = -1;
            end
        end
        
        function indices = getIndices(obj, posIdx, chanIdx)            
            indices = find(obj.imgNrAdapter{posIdx}(:,chanIdx) >= 0);            
        end
        
    end
    
    methods
        %% Constructor
        function obj = genericReadAsFrames(metafile, workingDir)
            
            
            if exist(strcat(workingDir,filesep,metafile),'file') ~= 2
                error('Metafile is not found. Aborting.');
            end
            
            metafileStr = fileread(strcat(workingDir,filesep,metafile));
            
            data = Json.getData(metafileStr);
            
            obj.absPath = workingDir;            
            obj.positions = unique(data.positions);
            obj.channels  = unique(data.channels);
            obj.croppingRegions = Json.getRoi(metafileStr, obj.channels);
            
            nPositions = length(obj.positions);
            nChannels = length(obj.channels);
            
            obj.positionsMap = containers.Map(obj.positions,1:nPositions);          
            
            obj.posChCounters = zeros(nPositions, nChannels);
            obj.posFrameCounters = zeros(nPositions, nChannels);
            
            obj.uniqueTimepoints = cell(nPositions,1);
            for i = 1:nPositions
                idx = strcmp(data.positions,obj.positions{i});
                obj.uniqueTimepoints{i} = unique(data.expectedTimepoints(idx));
            end
            
            for i = 1:nPositions
                nUniqueTimepoints = length(obj.uniqueTimepoints{i});
                obj.imgNrAdapter{i} = -ones(nUniqueTimepoints,nChannels);
                obj.exposureTimes{i} = zeros(nUniqueTimepoints,nChannels);
                obj.acquireTimes{i} = zeros(nUniqueTimepoints,nChannels);
            end
            
            for i = 1:length(data.positions)

                currExpectedTime = data.expectedTimepoints(i);
                posIdx = obj.positionsMap(data.positions{i});

                timepointIdx = ...
                    find(obj.uniqueTimepoints{posIdx} == currExpectedTime);
                
                chanIdx = obj.getChanIndex(data.channels{i});
                
                % disregard all doublett shots of frames.
                if obj.imgNrAdapter{posIdx}(timepointIdx,chanIdx) == -1
                    
                    obj.imgNrAdapter{posIdx}(timepointIdx,chanIdx) ...
                        = obj.posChCounters(posIdx,chanIdx);

                    obj.exposureTimes{posIdx}(timepointIdx,chanIdx) ...
                        = data.exposureTimes(i);

                    obj.acquireTimes{posIdx}(timepointIdx,chanIdx)...
                        = data.acquiredTimepoints(i);

                    obj.posFrameCounters(posIdx,chanIdx) = ...
                        obj.posFrameCounters(posIdx,chanIdx) + 1;

                end
                
                obj.posChCounters(posIdx,chanIdx) = ...
                    obj.posChCounters(posIdx,chanIdx) + 1;     
                     
            end
        end
        
        %% Accessors
        % Folder related functions
        function positions = getPositionList(obj, posRange)
            if nargin < 2
                positions = obj.positions;
            else
                positions = obj.positions(posRange);
            end
        end
        
        function chanIdx = getChanIndex(obj, chanName)
            chanIdx = find(strcmp(obj.channels,chanName));
        end
        
        % Get frame indices
        function indices = getIndicesByChanName(obj, posIdx, chanName)
            chanIdx = obj.getChanIndex(chanName);
            indices = getIndices(obj, posIdx, chanIdx);
        end
        
        function indices = getIndicesByPosAndChanName(obj, posName, chanName)
            posIdx = obj.positionsMap(posName);
            indices = obj.getIndicesByChanName(posIdx, chanName);
        end
        
        % Get images
        function img = getImgByChanName(obj, posIdx, frameIdx, chanName)
            chanIdx = obj.getChanIndex(chanName);
            img = obj.getImg(posIdx, frameIdx, chanIdx);
        end
        
        function img = getImgByPosAndChanName(obj, posName, frameIdx, chanName)
            posIdx = obj.positionsMap(posName);
            img = obj.getImgByChanName(posIdx, frameIdx, chanName);
        end

        function imgNr = getImgNr(obj, posIdx, frameIdx, chanName)
            chanIdx = obj.getChanIndex(chanName);
            imgNr = obj.imgNrAdapter{posIdx}(frameIdx,chanIdx);
        end

        % Get times
        function times = getAcquireTimes(obj, posName, chanName)
            chanIdx = obj.getChanIndex(chanName);
            posIdx = obj.positionsMap(posName);
            times = obj.acquireTimes{posIdx}(:,chanIdx);
        end
        
        function times = getUniqueTimesByChanName(obj, posName, chanName)
            posIdx = obj.positionsMap(posName);
            idx = obj.getIndicesByChanName(posIdx,chanName);
            times = zeros(size(obj.uniqueTimepoints{posIdx}));
            times(idx) = obj.uniqueTimepoints{posIdx}(idx);
        end
        
        function positionsMap = getPositionsMap(obj)
            positionsMap = obj.positionsMap;
        end
        
        function channels = getChannels(obj)
            channels = obj.channels;
        end
        
        % Other functions
        function croppingRegions = getCroppingRegions(obj, chanNames)
            if nargin == 2
                if ~iscell(chanNames)
                    chanName = chanNames;
                    chanIdx = obj.getChanIndex(chanName);
                    croppingRegions = obj.croppingRegions{chanIdx};
                else
                    nChannels = numel(chanNames);
                    croppingRegions = cell(1,nChannels);
                    for idx = 1:nChannels
                        chanName = chanNames(idx);
                        chanIdx = obj.getChanIndex(chanName);
                        croppingRegions{idx} = obj.croppingRegions{chanIdx};
                    end
                end
            else
                croppingRegions = obj.croppingRegions;
            end
        end
        
        %% Mutators
        function obj = setWorkingDirectory(obj, workingDir)
            obj.absPath = workingDir;
        end
    end
end
