function [valuesCy3, mask, bkg] = processArrays(dirCy3,dirOffset,parameters,bkgGrid)
% Performs analysis of specific operator binding experiments using
% microarrays.
%
% Input:
%   dirCy3 - cell array, each element is a path to directory with Cy3
%            fluorescence images.
%   dirOffset - path to directory with an offset Cy3 image, used for
%               background subtraction. If empty no background subtraction.
%   parameters - scructure array with fields:
%       overlap - number of tile overlapping pixels.
%       angularDisplacementTile - in degrees, rotate counter-clockwise
%                                 raw images (tiles).
%       angularDisplacement - in degrees, rotate counter-clockwise
%                             merged Cy3 images.
%       rangePositions - array with position indices in Cy3. If empty,
%                           all positions are used. Default [].
%       roi - cropping region in Cy3 images after rotation.
%       pxSize - pixel size in mum.
%       topLeftCorner - [x y] coordinates of the centroid of the
%                       (imaginary) spot in the top left corner of
%                       preprocessed image.
%   bkgGrid - compute average background using one of two preset masks.
%             Integer number bkgGrid defines the set and must be 1 or 2.
%             If 0, skip step. Optional, default 0.
%
% Output:
%   valuesCy3 - cell 2D array, each element corresponds to a microarray
%               spot and contains a vector of spot intensities in Cy3
%               sorted by image acquisition time.
%   mask - boolean matrix of microarray spots in the preprocessed images.
%
% Spartak Zikrin, Elf lab, 2021-09-12.
%% Parse parameters
if ~isfield(parameters,'rangePositions')
    parameters.rangePositions = [];
end
if nargin<4
    bkgGrid = 0;
end
assert(ismember(bkgGrid,[0 1 2]),'Set bkgGrid = 0,1 or 2.');
%% Preprocess offset image if it is available
if ~isempty(dirOffset)
    offset = stitchArrayImages(dirOffset,parameters.overlap,parameters.angularDisplacementTile,parameters.rangePositions,1);
    offset = preprocessImage(offset,parameters.angularDisplacement,parameters.roi);
else
    offset = 0;
end
%% Comput spot mask and background mask
imSize = parameters.roi([4 3])+1;
slide = zeros(imSize);
mask = arrayGridMask(slide,parameters.pxSize,parameters.topLeftCorner);
bkgMask = false(imSize);
if bkgGrid
    rowSpacing = 0.073323*1000/parameters.pxSize;
    props = regionprops(imtranslate(mask,[0 rowSpacing/2]),'Centroid');
    if bkgGrid==1
        for i=1:numel(props)
            if  props(i).Centroid(1) < imSize(2)*3/4 && props(i).Centroid(2) < imSize(1)*99/100 
                bkgMask(round(props(i).Centroid(2))-2:round(props(i).Centroid(2))+2,round(props(i).Centroid(1))-2:round(props(i).Centroid(1))+2)=true;
            end
        end
    elseif bkgGrid==2
        for i=1:numel(props)
            if  props(i).Centroid(1) > imSize(2)/2  && props(i).Centroid(2) < imSize(1)*99/100  && props(i).Centroid(2) > imSize(1)*1/4
                bkgMask(round(props(i).Centroid(2))-2:round(props(i).Centroid(2))+2,round(props(i).Centroid(1))-2:round(props(i).Centroid(1))+2)=true;
            end   
        end
    end
end
%% Preprocess Cy3 images together
angularDisplacement = parameters.angularDisplacement;
roi = parameters.roi;
tmpValuesCy3 = [];
bkg = [];
for i=1:numel(dirCy3)
    currSlidesCy3 = stitchArrayImages(dirCy3{i},parameters.overlap,parameters.angularDisplacementTile,parameters.rangePositions);
    if ~iscell(currSlidesCy3)
        currSlidesCy3 = {currSlidesCy3};
    end
    currTmpValuesCy3 =  cell(1,numel(currSlidesCy3));
    currBkg = zeros(1,numel(currSlidesCy3));
    parfor j=1:numel(currSlidesCy3)
        slideCy3 = preprocessImage(currSlidesCy3{j},angularDisplacement,roi);
        slideCy3 = slideCy3-offset;
        currTmpValuesCy3{j} = computeSpotIntensities(slideCy3,mask);
        if bkgGrid
            currBkg(j) = median(slideCy3(bkgMask));
        end
    end
    tmpValuesCy3 = [tmpValuesCy3 currTmpValuesCy3];
    bkg = [bkg currBkg];
end

%% Rearrange spot intensities array
[nRows,nCols] = size(tmpValuesCy3{1});
valuesCy3 = cell(nRows, nCols);
for i = 1:nRows
    for j = 1:nCols
        valuesCy3{i,j} = zeros(1,length(tmpValuesCy3));
        for k = 1:length(tmpValuesCy3)
            valuesCy3{i,j}(k)= tmpValuesCy3{k}(i,j);
        end 
    end
end