function [backBoneStruct] = computeSingleCellFit(inputImage,cellLabel,distanceThreshold)
% Function to compute the fit that goes through the cell and reprsent its
% backbone. 
% An important thing here is that some of the fits would be too poor (or 
% simply not possible to compute) if we kept the image constantly in the 
% same orientation: a banane shaped cell with both endpoints to the right
% could not have a second order polynomial function going through it,
% therefore, the fit is calculated in both normal orientation and also
% after rotating the image, then the fit with the lowest rmse if chosen to
% be the best.
%
% Inputs :
%     inputImage        : the current segmented image.
%     cellLabel         : the label of the cell we want to fit.
%     distanceThreshold : a threshold to choose which points inside the
%                         cell should be kept for curve fitting. This is a
%                         threshold over the distance from the border of
%                         the cell to the inside, in pixels.
% 
% Output :
%     backBoneStruct    : a structure containing some fields to store the
%                         fit of the backbone, a flag to know if the image
%                         had to be rotated, the length of the cell and the
%                         coordinates of the backbone of the cell (in
%                         pixels).

    if nargin < 3
        distanceThreshold = 3;
    end

    % Dilate the cell
    se = strel('disk',3,8);
    labelIm = imdilate(inputImage == cellLabel,se);
    
    % Compute the distance from the border of the cell to its inside
    D = bwdist(imcomplement(labelIm));
    threshD = D >= distanceThreshold;
    labelDist = threshD.*labelIm.*D;
    
    if max(max(labelDist)) == 0
        backBoneStruct.isRotated = -1;
        return
    end
    
    % Get the list of remaining pixels and their value and make the inner
    % pixels even more important (squared)
    labelStats = regionprops(labelIm,labelDist,'PixelList','PixelValues');
    PixList = vertcat(labelStats.PixelList);
    PixValues = vertcat(labelStats.PixelValues);

    options = fitoptions('poly2');
    options.Weights = PixValues.^3; 
    
    if ~isempty(PixList) 
        % Fit the points with a second order polynom
        [fitVal, fitErr] = fit(PixList(:,1),PixList(:,2),'poly2',options);
    else
        backBoneStruct.isRotated = -1;
        return
    end
    
    % Do the exact same thing after rotating the image
    rotatedIm = rot90(inputImage);
    rotatedLabel = imdilate(rotatedIm == cellLabel,se);
    rotD = bwdist(imcomplement(rotatedLabel));
    rotThreshD = rotD >= distanceThreshold;
    rotLabelDist = rotThreshD.*rotatedLabel.*rotD;
    rotLabelStats = regionprops(rotatedLabel,rotLabelDist,'PixelList','PixelValues');
    rotPixList = vertcat(rotLabelStats.PixelList);
    rotPixValues = vertcat(rotLabelStats.PixelValues);

    options = fitoptions('poly2');
    options.Weights = rotPixValues.^3; 

    if ~isempty(rotPixList)
        [rotFitVal, rotFitErr] = fit(rotPixList(:,1),rotPixList(:,2),'poly2',options);
    else
        backBoneStruct.isRotated = -1;
        return
    end
    
    % Keep the fit with lowest rmse
    if fitErr.rmse < rotFitErr.rmse
        % Not rotated image
        % Compute the coordinates of the points on the fit
        allCoord = fitVal(1:0.1:size(inputImage,2));
        roundedCoord = unique(round([[1:0.1:size(inputImage,2)]' allCoord]),'rows');
        roundedCoord(roundedCoord(:,2)<1,:) = [];
        roundedCoord(roundedCoord(:,1)<1,:) = [];
        roundedCoord(roundedCoord(:,2)>size(labelIm,1),:) = [];
        roundedCoord(roundedCoord(:,1)>size(labelIm,2),:) = [];
        linIdx = sub2ind(size(labelIm),roundedCoord(:,2),roundedCoord(:,1));
        fitIm = zeros(size(labelIm));
        
        if ~isempty(linIdx) && sum(isnan(linIdx)) == 0
            fitIm(linIdx) = 1;
        else
            backBoneStruct.isRotated = -1;
            return
        end
        
        fitIm(linIdx) = 1;
        % Compute the points inside the cells and that are on the fitting curve
        backBoneIm = (inputImage == cellLabel).*fitIm;
        backBoneTempCoord = find(backBoneIm);
        
        if isempty(backBoneTempCoord)
            backBoneStruct.isRotated = -1;
            return
        end
        
        [backBoneCoord(:,1), backBoneCoord(:,2)] = ind2sub(size(labelIm),backBoneTempCoord);
        
        [~,minIdx]= min(backBoneCoord(:,2));
        [~,maxIdx]= max(backBoneCoord(:,2));
        
        if backBoneCoord(minIdx,1) < backBoneCoord(maxIdx,1)
            backBoneCoord = sortrows(backBoneCoord,[1 2]);
        else
            backBoneCoord = sortrows(backBoneCoord,[1 -2]);
        end
        
        % Compute the length of the arc of the cell (cell length)
        arcLength = computeArcLength(fitVal,min(backBoneCoord(:,2))-0.5,max(backBoneCoord(:,2))+0.5);
        isRotated = false;
        backBoneStruct.Fit = fitVal;
    else
        % Same things but with rotated images
        allCoord = rotFitVal(1:0.1:max(size(rotatedIm,1)));
        roundedCoord = unique(round([[1:0.1:size(rotatedIm,1)]' allCoord]),'rows');
        roundedCoord(roundedCoord(:,2)<1,:) = [];
        roundedCoord(roundedCoord(:,1)<1,:) = [];
        roundedCoord(roundedCoord(:,1)>size(rotatedIm,2),:) = [];
        roundedCoord(roundedCoord(:,2)>size(rotatedIm,1),:) = [];
        linIdx = sub2ind(size(rotatedIm),roundedCoord(:,2),roundedCoord(:,1));
        fitIm = zeros(size(rotatedIm));
        
        if ~isempty(linIdx) && sum(isnan(linIdx)) == 0
            fitIm(linIdx) = 1;
        else
            backBoneStruct.isRotated = -1;
            return
        end
        
        backBoneIm = (inputImage == cellLabel).*rot90(fitIm,-1);
        backBoneTempCoord = find(backBoneIm);
        
        if isempty(backBoneTempCoord)
            backBoneStruct.isRotated = -1;
            return
        end
        
        [backBoneCoord(:,1), backBoneCoord(:,2)] = ind2sub(size(labelIm),backBoneTempCoord);
        
        [~,minIdx]= min(backBoneCoord(:,2));
        [~,maxIdx]= max(backBoneCoord(:,2));
        
        if backBoneCoord(minIdx,1) < backBoneCoord(maxIdx,1)
            backBoneCoord = sortrows(backBoneCoord,[1 2]);
        else
            backBoneCoord = sortrows(backBoneCoord,[1 -2]);
        end
        
        arcLength = computeArcLength(rotFitVal,min(backBoneCoord(:,1))-0.5,max(backBoneCoord(:,1))+0.5);
        isRotated = true;
        backBoneStruct.Fit = rotFitVal;
    end
    
    % Update the output structure with the computed values
    backBoneStruct.isRotated = isRotated;
    backBoneStruct.Coordinates = backBoneCoord;
    backBoneStruct.Length = arcLength;
end