function getMSDFit(finalTraj, unitTime, minTrajLength, maxTrajLength,frameSpacing)
% This function computes MSD for all trajectories and estimates MSD and
% plots the result
%
% Input:
%   - finalTraj : cell array of all trajectories
%   - unitTime : time between frames in seconds
%   - minTrajLength : minimal allowed length of trajectory
%   - maxTrajLenght : maximal allowed lenght of trajectory
%
% Output: 
%    Plot for MSD



% if we consider every frame
if nargin < 5
    frameSpacing = 1;
end

%change of time step
unitTime = unitTime*frameSpacing;

% Make MSD plot(one for all trajectories)
msdCell = cell(3, length(finalTraj));
%[max_size, ~] = max(cellfun('size', finalTraj, 1));
sqDisplTotCell = cell(3, maxTrajLength);

% Run through all trajectories
for ii = 1:length(finalTraj)
    tempTraj = finalTraj{ii}(1:frameSpacing:end, :)*0.08; %0.08 is the size of pixels in micro meters
    
    if size(tempTraj, 1)>=minTrajLength && size(tempTraj, 1)<=maxTrajLength
        % Calculate all squared displacements
        for dt = 1:size(tempTraj, 1)-1
            
            dCoords = tempTraj(1+dt:end,1:2) - tempTraj(1:end-dt,1:2);
            sqDispl = sum(dCoords.^2,2); % dx^2+dy^2
            
            sqDisplTotCell{1, dt} = [sqDisplTotCell{1, dt}; sqDispl];

            sqDisplX = dCoords(:, 1).^2;
            sqDisplTotCell{2, dt} = [sqDisplTotCell{2, dt}; sqDisplX];   
        end   
    end
end


% Make one general MSD list
msdTot = [length(sqDisplTotCell), 4];
msdTotX = [length(sqDisplTotCell), 4];


for dt = 1:length(sqDisplTotCell)
    msdTot(dt,1) = nanmean(sqDisplTotCell{1, dt}); % average
    msdTot(dt,2) = nanstd(sqDisplTotCell{1, dt}); % std
    msdTot(dt,3) = length(sqDisplTotCell{1, dt}); % n
    
    msdTotX(dt,1) = nanmean(sqDisplTotCell{2, dt}); % average
    msdTotX(dt,2) = nanstd(sqDisplTotCell{2, dt}); % std
    msdTotX(dt,3) = length(sqDisplTotCell{2, dt}); % n
end

% Calculate diffusion constant from the first 2 points

diffArr = [msdTot(1:2, 1)'];
xArr = [1, 2];
[p, ~] = polyfit(xArr.*unitTime, diffArr, 1);
diffOffset = p(2);
diffCoeff = p(1)/4;

diffArr = [msdTotX(1:2, 1)'];
xArr = [1, 2];
[pX, ~] = polyfit(xArr.*unitTime, diffArr, 1);
diffOffsetX = pX(2);
diffCoeffX = pX(1)/2;



% Plot the MSD if desired

color = 'r';
fighand1 = figure(666);
set(fighand1, 'Name', 'MSD plot','NumberTitle','off');
hold on
xArr = find(msdTot(:, 1));
fplot(strcat('x*', num2str(p(1)), '+', num2str(p(2))), [0, 5*unitTime], ['--' color]);
plot(xArr.*unitTime, msdTot(:, 1), ['-' color]);
errorbar(xArr.*unitTime, msdTot(:, 1), msdTot(:, 2)./(sqrt(msdTot(:, 3))-1), ['+' color]);
title('MSD XY');
xlabel('Time [s]');
ylabel('Mean square displacement [\mum^2]');
xlim([0, 20*unitTime]);
text(0.1, 0.9, ['Diff. Coeff = ', num2str(diffCoeff), '\mum^2/s'], 'Units', 'normalized');
text(0.1, 0.85, ['Offset = ', num2str(diffOffset), '\mum^2'], 'Units', 'normalized');
hold off


% fighand2 = figure(667);
% set(fighand2, 'Name', 'MSD plot','NumberTitle','off');
% hold on
% xArr = find(msdTotX(:, 1));
% fplot(strcat('x*', num2str(pX(1)), '+', num2str(pX(2))), [0, 5*unitTime], ['--' color]);
% plot(xArr.*unitTime, msdTotX(:, 1), ['-' color]);
% errorbar(xArr.*unitTime, msdTotX(:, 1)./(1e6), msdTotX(:, 2)./(sqrt(msdTotX(:, 3))-1), ['+' color]);
% title('MSD X');
% xlabel('Time [s]');
% ylabel('Mean square displacement [\mum^2]');
% xlim([0, 20*unitTime]);
% text(0.1, 0.9, ['Diff. Coeff = ', num2str(diffCoeffX), '\mum^2/s'], 'Units', 'normalized');
% text(0.1, 0.85, ['Offset = ', num2str(diffOffsetX), '\mum^2'], 'Units', 'normalized');
% hold off

end
