function file_struct=parseParameterFile(filename)
    % parse_file: parses a preferences file
    % usage: file_struct=parse_file(filename,keywords)
    %
    % arguments (input)
    % filename - character string containing the name of a
    % preferences file. If this file is not on
    % the path, then the name must also contain
    % the full path to that file.
    % keywords - cell array containing a list of legal
    % keywords. keywords are not case sensitive,
    % and only enough characters need be supplied
    % make the choice of keyword un-ambiguous.
    % defaults - (optional) cell array containing a list of
    % default values, one for each keyword supplied
    % if no defaults array is supplied, then no
    % defaults will be filled in. defaults must be
    % exactly the same size as keywords or defaults
    % must be empty or not even supplied
    %
    % arguments (output)
    % file_struct - structure containing one field for each
    % keyword found in the keyword list
    % 
    % parse_file ignores all empty lines, and all lines which
    % start with a '%' or '*' character. These are comments.
    % white space at the beginning of a line is also ignored.

    % Names of the different fields
    keywords={'noiseThreshold','scale','maxDist','minPoints','distanceThreshold', ...
              'sizeRoiInPhase','RoiInFluo','minWidth','maxWidth','minLength', ...
              'maxLength','minCellArea','segFirstFrame','stepFluo','timeStep', ...
              'minTrajLength','maxTrajLength','frameSpacing','trackCells', ...
              'rangePhaseImages','rangeFluoImages','BaxterSettingsPath', ...
              'useUTrack', 'uWindow', 'uMergeSplit', 'uMinTrackLen', ...
              'uMinSearchRadius', 'uMaxSearchRadius', 'uBrownStdMult', ...
              'uLinearMotion',...
              'radii', 'alpha', 'beta','radiusForThreshold','radSymThreshold',...
              'dotDetectionAlg', 'dotTrackingAlg'};
               %'minLengthPiece','timeStep','pixelSize','numberOfFittingPoints','Dimension'};

    % Defaults values of the different fields
    defaults={3.5, 80, 500, 5, 6, ...
              [860 860], [], 2, 25, 20, ...
              145, 70, 0, 1, 0.02, ...
              5, 50, 1, true, ...
              [], [], [], ...
              1, 1, 0, 2, ...
              2, 5, 3, ...
              1, ...
              [1 2 3 4 5], 1, 0, 2, 4, ...
              'wavelet', 'naive'};

    % check for defaults
    file_struct=[];
    if ~isempty(defaults)
      % stuff defaults
      file_struct=[];
      for i=1:length(defaults)
        file_struct=setfield(file_struct,keywords{i},defaults{i});
      end
    end
    
    if nargin<1
        return;
    end
    
    % start by checking that the file exists at all
    ex=exist(filename);
    if ex~=2
      error(['File not found: ',filename])
    end
    
    % open file
    fid=fopen(filename,'r');

    % get one record at a time
    flag=1;
    while flag
      rec=fgetl(fid);
      if ~ischar(rec)
        % then it must have been and end-of-file(-1)
        flag=0;
      else
        if ~isempty(rec)
          % ignore empty lines
          % grab the first token in the record
          [tok,rest]=strtok(rec,' ');
          % is this a comment (first token == '%')
          if ~strcmp(tok,'%')
            % check to see if this was a keyword in the list
            key=which_valid_property(tok,keywords);
            if ~isempty(key)
              % Remove the equal sign
              rest(strfind(rest,'='))=[];
              % stuff the field with the remainder of the record
              % after dropping any leading and trailing blanks.
              rest=strtrim(rest);
              k=find(rest~=' ');
              if isempty(k)
                rest='';
              elseif k(1)>1
                rest(1:(k-1))='';
              end
              try
                  rest1=str2num(rest);
              catch
                  % Not sure this works properly, hopefuly it is not needed
                  rest=cast(rest,'like',getfield(file_struct,key));
              end
              if isempty(rest1)
                file_struct=setfield(file_struct,key,rest);
              else
                file_struct=setfield(file_struct,key,rest1);
              end
            end
          end
        end
      end
    end

    % when all done, be nice and close the file
    fclose(fid);
end

% =================== begin sub-functions here ===================
% ================================================================
function [property,ind]=which_valid_property(property,valid_list)
    % checks the valid list of properties supplied, and returns the
    % one which matches, first looking for an exact match, then looking
    % to see if the user has only supplied the first few characters of
    % a valid property
    %
    % arguments (input):
    % property - string to be checked for validity 
    % valid_list - cell array containing valid property strings
    %
    % arguments (output):
    % property - string containing the full property name from valid_list
    % ind - scalar index of the property identified in valid_list

    ind=strmatch(lower(property),lower(valid_list),'exact');
    if isempty(ind)
      ind=strmatch(lower(property),lower(valid_list));
    end

    % is it in the list of legal properties?
    if isempty(ind)
      warndlg([property,' is not a valid option'])
      property='';
    elseif length(ind)>1
      warndlg([property,' matches more than one possible option.', ...
               ' be more specific.'])
      property='';
    else
      property=valid_list{ind};
    end
end