function registerBFToPhaseWithLandmarks(pathCroppedPhase,pathFluo,pathBright,firstFluo,stepFluo,firstCorrespondingPhase)
% Function to register fluorescence images to phase images and modify the cellLists accordingly.
% This writes out new fluorescence images that are in the space of the
% phase images. That way the meshes can be directly overlaid on the new
% fluorescence images. The transformed fluorescence images are saved in the
% same folder as the fluorescence images, with the same name, starting with
% "transformed_".
% Note that it is expected that the images are fairly close to each other
% already (phase images already cropped).
%
% Inputs:
%   pathCroppedPhase    : path to the cropped phase images.
%   pathFluo            : path to the fluorescence images.
%   pathBright          : path to the bright field images.
%   firstPhase          : index of the first fluorescence image.
%   stepFluo            : how often a fluorescence image is saved compared
%                         to a phase image.
% 
% Output: images are saved in pathFluo. No Matlab output.

    listCroppedPhase = dir(strcat(pathCroppedPhase,filesep,'*.tif*'));
    listFluo = dir(strcat(pathFluo,filesep,'*.tif*'));
    listBright = dir(strcat(pathBright,filesep,'*.tif*'));

    nrImages = length(listFluo);

%% Method using a combinaison of imregister and normxcorr2
% It turns out that using only imregister on all pair of images can lead to
% erros on some datasets, which require a lot of attention to fix. This
% method uses imregister only once, on the first pair of images (phase and
% BF) to obtain the scaling and rotation displacement. Then applies this
% transformation to all the other frames, and for thease frames, compute
% the normxcorr2 to get the translational offset between the phase and BF
% images. All transformation are then applied to the fluorescence images.

    for ii=firstFluo:nrImages
        currentPhase = imread(strcat(pathCroppedPhase,filesep,listCroppedPhase((ii-firstFluo)*stepFluo+1+firstCorrespondingPhase-1).name));
        origFluo = imread(strcat(pathFluo,filesep,listFluo(ii).name));
        currentFluo = imresize(flipdim(origFluo,1),size(currentPhase));
        currentBright = imresize(flipdim(imread(strcat(pathBright,filesep,listBright(ii).name)),1),size(currentPhase));

        if ii == firstFluo
            T = registerWithLandmarks(currentBright,currentPhase);
            
            outFluo = imwarp(currentFluo,affine2d(T),'OutputView',imref2d(size(currentPhase)),'FillValues',0);
            outBright = imwarp(currentBright,affine2d(T),'OutputView',imref2d(size(currentPhase)),'FillValues',0);
            
        else
            transformedBright = imwarp(currentBright,affine2d(T),'OutputView',imref2d(size(currentPhase)),'FillValues',0);
            
            c = normxcorr2(currentPhase,transformedBright);
            [~, imax] = max(abs(c(:)));
            [ypeak, xpeak] = ind2sub(size(c),imax(1));
            corr_offset = [(xpeak-size(currentPhase,2)) (ypeak-size(currentPhase,1))];
            transformedFluo = imwarp(currentFluo,affine2d(T),'OutputView',imref2d(size(currentPhase)),'FillValues',0);
            outFluo = zeros([size(transformedFluo,1)+corr_offset(1) size(transformedFluo,2)+corr_offset(2)]);
            outFluo = transformedFluo(1:(size(transformedFluo,1)-corr_offset(1))+corr_offset(1),1:(size(transformedFluo,2)-corr_offset(2))+corr_offset(2));
            
            outBright = zeros([size(transformedBright,1)+corr_offset(1) size(transformedBright,2)+corr_offset(2)]);
            outBright = transformedBright(1:(size(transformedBright,1)-corr_offset(1))+corr_offset(1),1:(size(transformedBright,2)-corr_offset(2))+corr_offset(2));
            
        end

        %figure, imshow(imfuse(currentPhase,outBright,'falsecolor','Scaling','joint','ColorChannels',[1 2 0]));
        %figure, imshow(imfuse(currentPhase,imresize(transformedFluo,[500 500]),'falsecolor','Scaling','joint','ColorChannels',[1 2 0]));

        imwrite(imresize(outFluo,size(currentPhase)),strcat(pathFluo,filesep,'transformed_',listFluo(ii).name));
        imwrite(imresize(outBright,size(currentPhase)),strcat(pathBright,filesep,'transformed_',listBright(ii).name));
    end
end

