function [T] = registerPhaseToBFWithLandmarks(pathCroppedPhase,pathBright,pathOutPhase,firstFluo,stepFluo, inTmatrix)
% Function to register phase images to fluorescence images.
% This writes out new fluorescence images that are in the space of the
% phase images. That way the meshes can be directly overlaid on the new
% fluorescence images. The transformed fluorescence images are saved in the
% same folder as the fluorescence images, with the same name, starting with
% "transformed_".
% Note that it is expected that the images are fairly close to each other
% already (phase images already cropped).
%
% Inputs:
%   pathCroppedPhase    : path to the cropped phase images.
%   pathFluo            : path to the fluorescence images.
%   pathBright          : path to the bright field images.
%   firstFluo           : index of the first fluorescence image.
%   stepFluo            : how often a fluorescence image is saved compared
%                         to a phase image.
% 
% Output: images are saved in pathFluo. No Matlab output.

listCroppedPhase = dir(strcat(pathCroppedPhase,filesep,'*.tif*'));
listBright = dir(strcat(pathBright,filesep,'*.tif*'));

if nargin == 6
    T = inTmatrix;
end

%% Method using a combinaison of imregister and normxcorr2
% It turns out that using only imregister on all pair of images can lead to
% erros on some datasets, which require a lot of attention to fix. This
% method uses imregister only once, on the first pair of images (phase and
% BF) to obtain the scaling and rotation displacement. Then applies this
% transformation to all the other frames, and for thease frames, compute
% the normxcorr2 to get the translational offset between the phase and BF
% images. All transformation are then applied to the fluorescence images.

for ii=firstFluo:length(listCroppedPhase)
    if ii == firstFluo && nargin < 6
        currentBright = imread(strcat(pathBright,filesep,listBright((ii-firstFluo)/stepFluo+1).name));
        currentPhase = imresize(imread(strcat(pathCroppedPhase,filesep,listCroppedPhase(ii).name)),size(currentBright));

        T = registerWithLandmarks(currentPhase,currentBright);

        outPhase = imwarp(currentPhase,affine2d(T),'OutputView',imref2d(size(currentBright)),'FillValues',0);

    elseif rem((ii-firstFluo),stepFluo)==0
        currentBright = imread(strcat(pathBright,filesep,listBright((ii-firstFluo)/stepFluo+1).name));
        currentPhase = imresize(imread(strcat(pathCroppedPhase,filesep,listCroppedPhase(ii).name)),size(currentBright));

        transformedPhase = imwarp(currentPhase,affine2d(T),'OutputView',imref2d(size(currentBright)),'FillValues',0);

        c = normxcorr2(currentBright,transformedPhase);
        [~, imax] = max(abs(c(:)));
        [ypeak, xpeak] = ind2sub(size(c),imax(1));
        corr_offset = [(xpeak-size(currentBright,2)) (ypeak-size(currentBright,1))];

        outPhase = zeros([size(transformedPhase,1)+corr_offset(1) size(transformedPhase,2)+corr_offset(2)]);
        outPhase = transformedPhase(1:(size(transformedPhase,1)-corr_offset(1))+corr_offset(1),1:(size(transformedPhase,2)-corr_offset(2))+corr_offset(2));

    else
        currentPhase = imresize(imread(strcat(pathCroppedPhase,filesep,listCroppedPhase(ii).name)),size(currentBright));

        transformedPhase = imwarp(currentPhase,affine2d(T),'OutputView',imref2d(size(currentBright)),'FillValues',0);

        c = normxcorr2(currentBright,transformedPhase);
        [~, imax] = max(abs(c(:)));
        [ypeak, xpeak] = ind2sub(size(c),imax(1));
        corr_offset = [(xpeak-size(currentBright,2)) (ypeak-size(currentBright,1))];

        outPhase = zeros([size(transformedPhase,1)+corr_offset(1) size(transformedPhase,2)+corr_offset(2)]);
        outPhase = transformedPhase(1:(size(transformedPhase,1)-corr_offset(1))+corr_offset(1),1:(size(transformedPhase,2)-corr_offset(2))+corr_offset(2));
    end

    %figure, imshow(imfuse(currentPhase,outBright,'falsecolor','Scaling','joint','ColorChannels',[1 2 0]));
    %figure, imshow(imfuse(currentPhase,imresize(transformedFluo,[500 500]),'falsecolor','Scaling','joint','ColorChannels',[1 2 0]));

    imwrite(imresize(outPhase,size(currentBright)),strcat(pathOutPhase,filesep,'transformed_',listCroppedPhase(ii).name));
end
end

