% [msd, sem, sd, n] = calcmsd(trajs, lag)
%
% Calculate MSD          Arvid Gynn, Oct 2014
% Takes a list of trajectories (with each entry in the trajectory
% describing a movement, not a position) and calculates the MSD, 
% using all position pairs with the given lag between. That is, also
% overlapping pairs. 
%
% The SEM is calculated based on the number of trajectories included, which
% gives a upper boud for the real SEM which is complicated when the
% displacements aren't independent. 
%
% Input:
% trajs         list of 2D trajectories
% lag           the lag interval to be used
%
% Output:
% msd           mean squared displacement
% sem           standard error of the mean, but see above 
% sd            standard deviation, but see above
% n             number of displacements used

function [msd, sem, sd, n] = calcmsd(trajs, lag)

disps_square = [];
in = 0;
out = 0;

% Loop over single trajectories
for i = 1:length(trajs)
    % Skip trajectories shorter than the chosen lag
    if lag > size(trajs{i}, 1)
        out = out + 1;
        continue;
    end
    
    % Since the trajs are stored as relative movements, calculate
    % cumulative displacements (a k a position relative start).
    pos = zeros(size(trajs{i}) + [1 0]);
    for j = 1:size(trajs{i}, 1)
        pos(j+1,:) = sum(trajs{i}(1:j,:), 1);
    end
    
    % Calculate MSDs
    l = pos(lag+1:end,:) - pos(1:end-lag,:);
    d_square = sum(l.^2, 2); % The row sums with each element squared
    
    % Store MSDs 
    disps_square(end+(1:length(d_square))) = d_square;
    in = in + 1; 
end

n   = length(disps_square);
msd = sum(disps_square)/n;
sd  = std(disps_square);
sem = sd/sqrt( in );

disp([num2str(in) ' trajectories included and ' num2str(out) ' skipped with lag ' num2str(lag) '.'])
end
