% ========================================================================= 
% imPipelineToFluoTrj, part of the trjbrowser suite for visualizing SPT
% trajectories. Copyright (C) 2015 Martin Lindén and Vladimir Curic
% E-mail: bmelinden@gmail.com, vladacuric@gmail.com
% =========================================================================
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or any
% later version.  This program is distributed in the hope that it will
% be useful, but WITHOUT ANY WARRANTY; without even the implied
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
% the GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
% =========================================================================
%
% function imPipelineToFluoTrj(trackingPath,trackingName,mvRange,imPipelinePath,getCellContours)
% read the output of image pipeline tracking data and convert it to cell
% vector format, one for each movie, in the form
%
% trackingPath  : path to trackedCells_*.mat files
% trackingName  : Name pattern of results files. For the most recent
%                 pipeline this could be 'outCells_*.mat' (default), while
%                 older versions used 'trackedCells_*.mat'.
% imPipelinePath: path to image analysis pipeline. Only needed if the
%                 pipeline is not already on the matlab path.
% mvRange       : range of segmentation files to convert. 
%                 Default or [] -> all movies. 
%                -1             -> select manually
% getCellContours : true/false indicator to extract cell contour or not.
%                   Default true.
% output: (the index m corresponds to experiment number mvRange(m) ).
%  M(m).movieFile  : absolute search path to tif stack
%  M(m).trj        : cell vector of trj, of the form [x y frame], in
%                    fluo space units.
% M(m).cellSegFile : absolute search path to the cell segmentation image
% M(m).cellOutlines: cell vector of cell outlines, in fluo space units 
%
% ML 2015-10-30
function M=imPipelineToFluoTrj(trackingPath,trackingName,mvRange,imPipelinePath,getCellContours)

if( ~exist('trackingPath','var') || ~exist(trackingPath,'dir'))
    trackingPath= uigetdir(pwd, 'Select tracking data folder');
end
if( ~exist('trackingName','var') || isempty(trackingName))
    trackingName= 'outCells_*.mat';
end

if(exist('imPipelinePath','var') && ~isempty(imPipelinePath))
    addpath(genpath(imPipelinePath));
end
if(~exist('getCellContours','var') || isempty(getCellContours))
    getCellContours=true;
end

% compute related paths and find the actual files
cellSegPath = fullfile(trackingPath,'SegmentedImages');
fluoPath    = fullfile(trackingPath,'..','Fl');

trackingFile = dir(fullfile(trackingPath,trackingName));
cellSegFile  = dir(fullfile(cellSegPath,'*tif'));
fluoFile     = dir(fullfile(fluoPath,'*tif'));

if(~exist('mvRange','var') || isempty(mvRange) )
    mvRange=1:numel(fluoFile);
elseif(mvRange==-1)
    names=cell(1,numel(fluoFile));
    for k=1:numel(fluoFile)
        [~,a,b]=fileparts(fluoFile(k).name);
        names{k}=[a b];
    end
    [mvRange,~] = listdlg('ListString',names,'SelectionMode','single',...
        'Name','select experiment','ListSize',[300 400]);
end

M=struct('trj',{cell(0)},'cellOutlines',{[]},'cellOrientation',{[]},...
    'xySeg2Fluo',{[]},'cellRegionProps',{[]},...
    'movieFile',{''},'cellSegFile',{''},'trjFile',{''},...
    'moviePath',{''},'cellSegPath',{''},'trjPath',{''});
%for m=1:numel(trackingFile);
 for n=1:numel(mvRange);   
    m=mvRange(n);
    % load the data for this positions
    M(n).moviePath   =fluoPath;
    M(n).movieFile   =fullfile(fluoPath,fluoFile(m).name);
    M(n).cellSegPath =cellSegPath;
    M(n).cellSegFile =fullfile(cellSegPath,cellSegFile(m).name);

    M(n).trj=cell(0);    
    M(n).cellOutlines=cell(0);    

    M(n).trjPath   =trackingPath;
    M(n).trjFile   =fullfile(trackingPath,trackingFile(m).name);

    
    TRJ=   load(M(n).trjFile);
    TRJ=TRJ.cellsArray;

    % read segmentation
    CS=imread(fullfile(cellSegPath,cellSegFile(m).name));

    % cell segmentation properties
    posProps= regionprops(CS,'Orientation','BoundingBox','Centroid');
    M(n).cellRegionProps=posProps;
    % compute scale factors
    % Fluo movie size
    FlInfo = imfinfo(fullfile(fluoPath,fluoFile(m).name));
    FlColums = FlInfo(1).Width;
    FlRows = FlInfo(1).Height;

    % to convert segmentation coordinates to fluo coordinates
    yseg2Fluo=FlRows/size(CS,1);
    xseg2Fluo=FlColums/size(CS,2);
    M(n).xySeg2Fluo=[xseg2Fluo yseg2Fluo];
    
    for c=1:numel(TRJ)
        if(getCellContours)
            % extract contours
            B = bwboundaries(CS==c);
            for k=1:length(B)
                M(n).cellOutlines{end+1}=[xseg2Fluo*B{k}(:,2) yseg2Fluo*B{k}(:,1)];
            end
        end
        % loop over trajectories
        for p=1:numel(TRJ(c).particles)
            traj_seg=TRJ(c).particles(p).trajImageCoord;
            frames=TRJ(c).particles(p).firstFrame-1+(1:TRJ(c).particles(p).numberOfFrame); 
            cellId=c*ones(size(frames'));
            M(n).trj{end+1}=[xseg2Fluo*traj_seg(:,1) yseg2Fluo*traj_seg(:,2) frames' cellId];
        end
    end    
end
