% ========================================================================= 
% initializeTrjBrowser, part of the trjbrowser suite for visualizing SPT
% trajectories. Copyright (C) 2015 Martin Lindén and Vladimir Curic
% E-mail: bmelinden@gmail.com, vladacuric@gmail.com
% =========================================================================
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or any
% later version.  This program is distributed in the hope that it will
% be useful, but WITHOUT ANY WARRANTY; without even the implied
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
% the GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
% =========================================================================
%
% trj=initializeTrjBrowser('par1',value1,value2,...,'par2',value1,value2,...)
% internal function to initialize the trj objects used by trjBrowser.m. 
% A call to trjBrowser('par1',value1,value2,...,'par2',value1,value2,...)
% to passes on all arguments to initializeTrjBrowser, but also starts the
% GUI. See help trjBrowser for arguments to pass.
%
% output: 
% trj - a struct with all kinds of trajectory and plot information
% used to display the trajectory and movie.


% type help trjBrowser for usage instructions.
% ML&VC 2014-12-xx
function trj=initializeTrjBrowser(varargin)
%% parse input parameters and ask for file names if needed
%% parse input argument and extract some trj fields
% trj.tifStackName=tifStackFullPath;
% trj.mvdata=ML_loadStack2(tifStackFullPath);
% trj.maxFrame=size(trj.mvdata,3); % numer of frames in the movie
% disp(['done! Found ' int2str(trj.maxFrame) ' frames.'])

v=0; % varargin index
trj=struct;
lastFolder=pwd;
fillGaps=false;
while(v<nargin)
   v=v+1;
   param=varargin{v};
   if(strcmp(param,'tif file'))
       v=v+1;
       tifStackFullPath=varargin{v};
       if(isempty(tifStackFullPath))
           % select tif stack manually if not given
           [tifStackFile, tifStackPath] = uigetfile({'*.tif','*.tiff'}, 'Select movie file (.tif, .tiff)',lastFolder);
           tifStackFullPath=fullfile(tifStackPath,tifStackFile);
           lastFolder=tifStackPath;
       end
       if(~exist(tifStackFullPath,'file'))
           error(['No tif file selected, or no file found at ' tifStackFullPath ])
       end
       % should be OK: load tif stack
       trj.tifStackName=tifStackFullPath;
       trj.mvdata=ML_loadStack2(tifStackFullPath);
       trj.maxFrame=size(trj.mvdata,3); % numer of frames in the movie
       disp(['Read tif stack with ' int2str(trj.maxFrame) ' frames.'])       
   elseif(strcmp(param,'tif stack'))
       v=v+1;
       trj.mvdata=varargin{v};
       trj.maxFrame=size(trj.mvdata,3); % numer of frames in the movie
       disp(['Read tif stack with ' int2str(trj.maxFrame) ' frames.'])
       trj.tifStackName='';       
   elseif(strcmp(param,'trj file'))
       v=v+1;
       trjFileFullPath=varargin{v};
       if(isempty(trjFileFullPath))
           [trjFileName, trjFilePath] = uigetfile({'*.mat'}, 'Select trajectory file (.mat)','lastFolder');
           trjFileFullPath=fullfile(trjFilePath,trjFileName);
           lastFolder=trjFilePath;
       end
       if(~exist(trjFileFullPath,'file'))
           error(['No .mat file found at ' trjFileFullPath])
       end
       % should be OK: load trajectory file       
       v=v+1;
       trjField=varargin{v};
       if(isempty(trjField)) % then select a field manually
           vars=whos('-file',trjFileFullPath);       
           [Selection,~] = listdlg('ListString',{vars.name},'SelectionMode','single',...
               'Name','select trajectory variable','ListSize',[300 400]);
           trjField=vars(Selection).name;
       end
       trjdata=load(trjFileFullPath,trjField);
       trj.trj=trjdata.(trjField);

       trj.trjFileName=trjFileFullPath;
       trj.trjFileField=trjField;
   elseif(strcmp(param,'trj'))
       v=v+1;
       trj.trj=varargin{v};
       if(~iscell(trj.trj))
          error('The parameter trj must be followed by a trajectory cell vector.')
       end
   elseif(strcmp(param,'trackPathOld'))
       v=v+1;
       trackingPath=varargin{v};
       if(isempty(trackingPath))
          trackingPath= uigetdir(lastFolder, 'Select tracking data folder'); 
          lastFolder=trackingPath;
       end
       
       v=v+1;
       movieNumber=varargin{v};
       if(isempty(movieNumber))
           movieNumber=-1;
       end
       M=imPipelineToFluoTrj(trackingPath,'trackedCells_*.mat',movieNumber);       
       trj.trj=M.trj;
       trj.tifStackName=M.movieFile;       
       trj.mvdata=ML_loadStack2(trj.tifStackName);
       trj.maxFrame=size(trj.mvdata,3); % numer of frames in the movie
       disp(['Read tif stack with ' int2str(trj.maxFrame) ' frames.'])    
       
       trj.cellOutlines=M.cellOutlines;
   elseif(strcmp(param,'trackPath'))
       v=v+1;
       trackingPath=varargin{v};
       if(isempty(trackingPath))
          trackingPath= uigetdir(lastFolder, 'Select tracking data folder'); 
          lastFolder=trackingPath;
       end
       
       v=v+1;
       movieNumber=varargin{v};
       if(isempty(movieNumber))
           movieNumber=-1;
       end
       M=imPipelineToFluoTrj(trackingPath,'outCells_*.mat',movieNumber);       
       trj.trj=M.trj;
       trj.tifStackName=M.movieFile;       
       trj.mvdata=ML_loadStack2(trj.tifStackName);
       trj.maxFrame=size(trj.mvdata,3); % numer of frames in the movie
       disp(['Read tif stack with ' int2str(trj.maxFrame) ' frames.'])    
       
       trj.cellOutlines=M.cellOutlines;
   elseif(strcmp(param,'segImage'))
       v=v+1;
       CS=varargin{v};
       if(~isfield(trj,'mvdata'))
           error('Must add segImage after fluo-movie in order to compute scale factors.')
       end
       disp('Creating new cell outlines from segmentation image')
       [FlRows,FlColums,~]=size(trj.mvdata);
       % to convert segmentation coordinates to fluo coordinates
       yseg2Fluo=FlRows/size(CS,1);
       xseg2Fluo=FlColums/size(CS,2);
       trj.cellOutlines=cell(0);
       for c=1:max(CS(:)) % loop over all segmented cells
           % extract contours
           B = bwboundaries(CS==c);
           for k=1:length(B)
               trj.cellOutlines{end+1}=[xseg2Fluo*B{k}(:,2) yseg2Fluo*B{k}(:,1)];
           end
       end       
       clear CS B yseg2Fluo xseg2Fluo
   elseif(strcmp(param,'states'))
       v=v+1;
       states=varargin{v};
   elseif(strcmp(param,'gapfill'))
       fillGaps=true;
   else   
       error(['parameter ' param ' not recognized.'])
   end
end

%% initialize the figure and trj object

% replace NaNs with linear interpolation
if(fillGaps)
   for t=1:numel(trj.trj)
        isFin=find(isfinite(trj.trj{t}(:,1)));
        isGap=find(~isfinite(trj.trj{t}(:,1)));
       
        trj.trj{t}(isGap,1)=interp1(isFin,trj.trj{t}(isFin,1),isGap,'linear');
        trj.trj{t}(isGap,2)=interp1(isFin,trj.trj{t}(isFin,2),isGap,'linear');
   end
end

%trj.positionHandle=plot(rand(1,5),rand(1,5),'m*');
trj.frameRadius=5;
numTrj=length(trj.trj);
trj.trjFutureColor=[0.5*rand(numTrj,1) ones(numTrj,1) 0.5*rand(numTrj,1)];
%ind=rand(numTrj,1)>0.5;
%trj.trjFutureColor(ind,:)=trj.trjFutureColor(ind,[2 1 3]); % röd-grön röra

% 
%%%trj.trjPastColor  =[ones(numTrj,1) rand(numTrj,2) ];
trj.trjPastColor  =[ones(numTrj,1) zeros(numTrj,2) ];

if(exist('states','var'))
    Smax=0;
    for t=1:length(states)
       Smax=max(Smax,max(states{t}));
    end    
    trj.states=states;    
    trj.Smax=Smax;
    trj.stateColor=[linspace(0,1,Smax)' zeros(Smax,1) linspace(1,0,Smax)' ];
    trj.stateColor=[linspace(1,0,Smax)' linspace(0,1,Smax)' linspace(0.99,1,Smax)'];
    
end

% initialize movie plot
figure;
trj.mvFigure=gcf;
trj.imHandle=imagesc(trj.mvdata(:,:,1));
colormap gray
axis equal
axis tight
axis off

hold on
if(isfield(trj,'cellOutlines'))
    for m=1:numel(trj.cellOutlines)
        plot(trj.cellOutlines{m}(:,1),trj.cellOutlines{m}(:,2),...
            'color',[1 0 0],'linew',2);
    end
end

trj.plotHandle=[];

% set constant color range based on the mid fram of the movie
tMid=round(size(trj.mvdata,3));
trj=trjBrowser_updateFrame(trj,tMid);
clim=get(gca,'clim');
set(gca,'clim',clim);

trj=trjBrowser_updateFrame(trj,1);

% for ii = 1:length(trj.trj)
%     tmpTraj = trj.trj{1,ii};
%     [sz_points, ~] = size(tmpTraj);
%     firstFrame = tmpTraj(1,3); %%% first frame in the trajectory
%     lastFrame = tmpTraj(sz_points,3); %%% last frame in the trajectory
%     
%     %% check if any trajectory exists for the current frame
%     indexCurrentFrame = (firstFrame<=1).*(lastFrame>=1);
%   
%     %% plot the trajectory to the current frame
%     if indexCurrentFrame == 1
%         %% loop over all points in the trajectory
%         for jj = 1:find(tmpTraj(:,3)==1);
%             x = tmpTraj(1:jj,1)';
%             y = tmpTraj(1:jj,2)';
%         end
%         plot(x,y,'color',[1,rand(1,2)])
%         hold on
%     end
%     
% end %%% ii




