% ========================================================================= 
% trjBrowser.m, tool for visualizing SPT trajectories.
% Copyright (C) 2015 Martin Lindén and Vladimir Curic
% E-mail: bmelinden@gmail.com, vladacuric@gmail.com
% =========================================================================
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or any
% later version.  This program is distributed in the hope that it will
% be useful, but WITHOUT ANY WARRANTY; without even the implied
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
% the GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
% =========================================================================
%
% trjBrowser([],'par1',value1,value2,...,'par2',value1,value2,...)
%
% The trjBrowser GUI displays a fluorescent movie, cell outlines (if
% present), and trajectories on top of each other. AVI movies can also be
% produced. 
%
% input is given as sets of parameter names and values. In many cases,
% empty values will launch a GUI that lets the use choose manually. 
%
% 'tif file' , path : full path to tif stack. If path=[], then the file is
%                     selected from a gui. 
% 'tif stack', MV : when an actual tif stack (a*b*c matrix) is passed.
% 'trj file', file, field : trajectory stored in a .mat file as a
%                           variable named field. Both file and field can
%                           be empty vectors.
% 'trj', trj : an actual trajectory cell vector is passed. Both here, and
%              with the 'trj file' option, the trajectory should be a cell
%              vector of column vectors, on the form 
%                 x{k} = [ . . .        ;
%                          x1 y1 frame1 ;
%                          x2 y2 frame2 ;
%                          x3 y3 frame3 ;
%                          . . . ], 
%              where xN,yN are point coordinates, and frameN refer to frame
%              numbers in the tifstackfile. Frames should probably be
%              consecutive (but this is not enforced). Missed points (some
%              trajectory builders infer them) will be indicated in green
%              if the coordinates are NaNs, e.g., 
%                 x{k}{m,:} = [ NaN NaN frameM].
% 'segImage',CS     : CS = segmentation image (background=0, and each cell
%                     is a region with its own integer valöue). This adds
%                     cell outlines to the image, overwriting outlines from
%                     earlier inputs. Must be given after a movie file has
%                     soimehow been added. 
% 'trackPath',path,m    : path = folder to pipeline tracking results, with
%                         outCells* naming convention, i.e., the folder
%                         with the outCells_xxx.mat files.
%                         m=movie number to use. Both can be selected
%                         manually if empty matrices are given.
% 'trackPathOld',path,m : same as trackPath, but with the trackedCells*
%                         naming convention. 
% 'states', st            : color by segmented state, as given by state cell
%                         vector s{k}(t) = state in interval [t t+1] in trj k
% 'gapfill'         : If given, gaps (NaN coordinates) are filled out by
%                     linear interpolation.
%                     
% NOTE: the image analysis pipeline must be in the matlab path for this
% option to work. 
%
% Examples 
% --------
% Load the test data set from the trjBrowserStart folder:
% trjBrowser('tif file','testdata/test7.tif','trj file','testdata/Trajectories_for_test7.mat','Coordinates')
% 
% select a tif fil and trajectory file graphically:
% trjBrowser([],'tif file',[],'trj file',[],[])
%
% Use the first movie from an image analysis pipeline output:
% trjBrowser([],'trackPath',[],1);
%
% Use the first movie from an image analysis pipeline output, old naming
% convention:
% trjBrowser([],'trackPathOld',[],1);
%
% ----------------------------------------------
% NOTE: New syntax with empty first argument [].
% ----------------------------------------------
% Martin Lindén, bmelinden@gmail.com, 2016-01-11

function varargout = trjBrowser(varargin)

% TRJBROWSER MATLAB code for trjBrowser.fig
%      TRJBROWSER, by itself, creates a new TRJBROWSER or raises the existing
%      singleton*.
%
%      H = TRJBROWSER returns the handle to a new TRJBROWSER or the handle to
%      the existing singleton*.
%
%      TRJBROWSER('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in TRJBROWSER.M with the given input arguments.
%
%      TRJBROWSER('Property','Value',...) creates a new TRJBROWSER or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before trjBrowser_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to trjBrowser_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help trjBrowser

% Last Modified by GUIDE v2.5 28-Jan-2016 21:32:34

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @trjBrowser_OpeningFcn, ...
                   'gui_OutputFcn',  @trjBrowser_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before trjBrowser is made visible.
function trjBrowser_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to trjBrowser (see VARARGIN)

% Choose default command line output for trjBrowser
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% get GUI data structure
h=guidata(hObject);

% read data
%tifStackName=varargin{1};trjFileName=varargin{2};
%h.trjObject=initializeTrjBrowser(tifStackName,trjFileName);
h.trjObject=initializeTrjBrowser(varargin{2:end});

% set slider properties
% slider-step seems to use a normalized scale from 0 to 1, and the 'Value'
% interpolates linearly between Min and Max.
set(h.slider1,'SliderStep',[1/(h.trjObject.maxFrame-1) 0.2]) 
set(h.slider1,'Min',1);
set(h.slider1,'Max',h.trjObject.maxFrame);
set(h.slider1,'Value',1);
h.uibuttongroup1.Title=['frame : ' num2str(get(h.slider1,'Value')) ];

% save new GUI data
guidata(hObject,h);

% UIWAIT makes trjBrowser wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = trjBrowser_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on slider movement.
function slider1_Callback(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider

% round slider value to integer (frame)
frame=round(get(hObject,'Value'));
set(hObject,'Value',frame);

% display frame number
h=guidata(hObject);
h.uibuttongroup1.Title=['frame : ' int2str(frame)];

% update which frame to display
h.trjObject=trjBrowser_updateFrame(h.trjObject,frame);
guidata(hObject,h);

% --- Executes during object creation, after setting all properties.
function slider1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to slider1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes on button press in makeMovieButton.
function makeMovieButton_Callback(hObject, eventdata, handles)
% hObject    handle to makeMovieButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
h=guidata(hObject);
frame0=round(get(h.slider1,'Value'));
h.trjObject=trj2aviMovie(h.trjObject);

h.trjObject=trjBrowser_updateFrame(h.trjObject,frame0);
guidata(hObject,h);


% --- Executes on button press in closeButton.
function closeButton_Callback(hObject, eventdata, handles)
% hObject    handle to closeButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
h=guidata(hObject);
try
    delete(h.trjObject.mvFigure)
catch
end
try
    delete(h.figure1)
catch
end


% --- Executes during object deletion, before destroying properties.
function makeMovieButton_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to makeMovieButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object deletion, before destroying properties.
function figure1_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
h=guidata(hObject);
try
    delete(h.trjObject.mvFigure)
catch
end
try
    delete(h.figure1)
catch
end


