% ========================================================================= 
% trjKymograph.m, tool for visualizing SPT trajectories as part of the 
% trjbrowser suite.
% 
% Copyright (C) 2015 Martin Lindén E-mail: bmelinden@gmail.com
% =========================================================================
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or any
% later version.  This program is distributed in the hope that it will
% be useful, but WITHOUT ANY WARRANTY; without even the implied
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
% the GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
% =========================================================================
% 
% trj=trjKymograph(tifstackfil,trjfile,trjfilefield)
%
% plot kymographs of movies with identified trajectories on top, which is a
% good way to get a graphical overview of how well the dots in a SPT
% experiment have been connected. 
% 
% Input (if no input is given, the user can give the input via GUIs
% instead):
%
% tifstackfile  : a tif movie
% trjfile       : a .mat file with trajectory information. This should be a
%                 cell vector of column vectors, on the form
%                 x{k} = [ . . .        ;
%                          x1 y1 frame1 ;
%                          x2 y2 frame2 ;
%                          x3 y3 frame3 ;
%                          . . . ], where xN,yN are point coordinates, and
%                          frameN refer to frame numbers in the
%                          tifstackfile. Frames should probably be
%                          consecutive (but this is not enforced). 
%                          Missed points (some trajectory builders infer 
%                          them) will be indicated in green if the
%                          coordinates are NaNs, e.g., 
%                          x{k}{m,:} = [ NaN NaN frameM]
% trjfilefield  : name of the variable in the trjfile that contains the
%                 trajectory cell vector.
% 
% M.L. 2014-12-03. (inspired by Paul Wiggins).
% 

%% start of actual code
function trj=trjKymograph(varargin)


%% parse input parameters and ask for file names if needed
% read and interpret input arguments
% one may want to be more flexible about loading the data, for example if
% one mat-file contains several variables, one could choose which one

% parse varargin
if(~isempty(varargin))
    tifStackFullPath=varargin{1};
    trjFileName=varargin{2};
    trjField=varargin{3};
    trjFileFullPath=fullfile(pwd,trjFileName);
else
    % select inputs graphically if not given
    [tifStackFullPath, tifStackPath] = uigetfile({'*.tif','*.tiff'}, 'Select movie file (.tif, .tiff)');
    tifStackFullPath=fullfile(tifStackPath,tifStackFullPath);
    
    [trjFileName, trjFilePath] = uigetfile({'*.mat'}, 'Select trajectory file (.mat)');
    trjFileFullPath=fullfile(trjFilePath,trjFileName);
    vars=whos('-file',trjFileFullPath);
    [Selection,~] = listdlg('ListString',{vars.name},'SelectionMode','single',...
        'Name','select trajectory variable','ListSize',[300 400]);
    trjField=vars(Selection).name;
end
%% compute kymographs
% plot parameters
trj=struct;
% open the figure for showing the movie
figure;
trj.kymFigure=gcf;

% store movie origin
trj.tifStackName=tifStackFullPath;
trj.trjFileName=trjFileFullPath;
trj.trjFileField=trjField;

% save image data
disp('loading movie data . . .')
trj.mvdata=ML_loadStack2(tifStackFullPath);
trj.maxFrame=size(trj.mvdata,3); % numer of frames in the movie
disp(['done! Found ' int2str(trj.maxFrame) ' frames.'])

trjdata=load(trjFileFullPath,trjField);
trj.trj=trjdata.(trjField);

%% compute kymographs
[nx,ny,nt]=size(trj.mvdata);
% 1: using mean values
%kx=cell(1,2);
%ky=cell(1,2);

%ky{1}=reshape(mean(trj.mvdata,2),nx,nt);
%kx{1}=reshape(mean(trj.mvdata,1),ny,nt);

% 2: using smoothing and max
ky=reshape(max(smooth3(trj.mvdata,'box',[1 3 1]),[],2),nx,nt);
kx=reshape(max(smooth3(trj.mvdata,'box',[3 1 1]),[],1),ny,nt);

% 3: using raw max
%ky{3}=reshape(max(smooth3(trj.mvdata,'box',[1 3 1]),[],2),nx,nt);
%kx{3}=reshape(max(smooth3(trj.mvdata,'box',[3 1 1]),[],1),ny,nt);

%% plot them
figure

subplot(2,1,1)
imagesc(kx)
colormap gray
hold on
legh=zeros(1,2); % handles to legend entries

for kk=1:length(trj.trj)
    legh(1)=plot(trj.trj{kk}(:,3), trj.trj{kk}(:,1),'r','linew',1);
    legh(2)=plot(trj.trj{kk}([1 end],3), trj.trj{kk}([1 end],1),'ro','markersize',3,'markerface','r');
    % bridge single-step gaps 
    ind=find(isnan(trj.trj{kk}(:,1)));
    if(~isempty(ind))
       for nn=1:length(ind)
          try
             i1i2=ind(nn)+[-1 1];
             legh(3)=plot(trj.trj{kk}(i1i2,3), trj.trj{kk}(i1i2,1),'g','linew',1);
          catch
              
          end
       end        
    end    
end
xlabel('t / \Delta t')
ylabel('x [px]')
% add legend
if(length(legh)==2)
    legend(legh,'trj','trj ends')
else
    legend(legh,'trj','trj ends','missed pt')
end
subplot(2,1,2)
imagesc(ky)
colormap gray
hold on
for kk=1:length(trj.trj)
    plot(trj.trj{kk}(:,3), trj.trj{kk}(:,2),'r','linew',1)
    plot(trj.trj{kk}([1 end],3), trj.trj{kk}([1 end],2),'ro','markersize',3,'markerface','r')
        % bridge single-step gaps 
    ind=find(isnan(trj.trj{kk}(:,1)));
    if(~isempty(ind))
       for nn=1:length(ind)
          try
             i1i2=ind(nn)+[-1 1];
             plot(trj.trj{kk}(i1i2,3), trj.trj{kk}(i1i2,2),'g','linew',1)
          catch
              
          end
       end        
    end    

end
xlabel('t / \Delta t')
ylabel('y [px]')



